/***************************************************************************

                          colorsprite.cpp
                          a sprite for SDL-Apps that consists just of
                          color, without any images/textures - Implementation
                             -------------------
    begin                : Mon Mai 26 2003
    copyright            : (C) 2003 by Thomas Unterthiner
    email                : thomas_unterthiner@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "colorsprite.h"

// -------------------------
// Ctor/Dtor
// -------------------------


ColorSprite::ColorSprite(int PosX, int PosY, int Width, int Height,
                         Uint32 Color, Uint32 BackgroundColor)
/******************************************************************************
 * the normal Ctor, initiates all Members.
 *****************************************************************************/
            : mColor(Color),
              mRedrawColor(BackgroundColor),
              mPositionChanged(false)
{
	// can't set coordinates with setPos/setSize in the Ctor, because
	// that would also initiate mRedrawCoordinates with wrong parameters
	// by calling saveLastDrawnPosition()
	mCoordinates.x = PosX;
	mCoordinates.y = PosY;
	mCoordinates.w = Width;
	mCoordinates.h = Height;
}

ColorSprite::ColorSprite(const ColorSprite& rhs)
            : mColor(rhs.mColor),
              mRedrawColor(rhs.mRedrawColor)            
/******************************************************************************
 * copy-ctor
 * WATCH OUT: doesn't copy the redraw-information
 *****************************************************************************/
{
	// can't set coordinates with setPos/setSize in the Ctor, because
	// that would also initiate mRedrawCoordinates with wrong parameters
	// by calling saveLastDrawnPosition()
	mCoordinates.x = rhs.mCoordinates.x;
	mCoordinates.y = rhs.mCoordinates.y;
	mCoordinates.w = rhs.mCoordinates.w;
	mCoordinates.h = rhs.mCoordinates.h;
}

ColorSprite::~ColorSprite()
/******************************************************************************
 * Deconstructor, doesn't anything
 *****************************************************************************/
{
	
}

 

// -------------------------
// public members
// -------------------------


void ColorSprite::draw(SDL_Surface* Surface)
/******************************************************************************
 * draws the Sprite on the surface, and - if neccessary - it cancels the old
 * image of the sprite from the surface, by redrawing all old sprite-images
 * with mRedrawColor
 * 	PARAMETER
 * 		SDL_Sprite* Surface: the Surface to draw into
 *****************************************************************************/
{
	if (mPositionChanged)
		redraw(Surface);

	SDL_FillRect(Surface, &mCoordinates, mColor);
	mPositionChanged = false;
}



void ColorSprite::move(int ByX, int ByY)
/******************************************************************************
 * moves the sprite by a certain amount of pixel.
 * Remembers the old position of the sprite by adding a new entry to
 * mRedrawPositions. These Positions will than be overdrawn with mRedrawColor
 * (meaning they'll be deleted from the Surface) on the next call of
 * ColorSprite.draw()
 *
 * 	PARAMETER
 * 		int ByX, ByY: move the sprite by so many X/Y Pixel
 *****************************************************************************/
{
	saveLastDrawnPosition();
	mCoordinates.x += ByX;
	mCoordinates.y += ByY;
}



// -------------------------
// protected members
// -------------------------


void ColorSprite::setPos(int PosX, int PosY)
/******************************************************************************
 * moves the sprite to a certain position.
 * Remembers the old position of the sprite by adding a new entry to
 * mRedrawPositions. These Positions will than be overdrawn with mRedrawColor
 * (meaning they'll be deleted from the Surface) on the next call of
 * ColorSprite.draw()
 *
 * 	PARAMETER
 * 		int PosX, PosY: new Coordinates of the sprite.
 *****************************************************************************/
{
	saveLastDrawnPosition();
	mCoordinates.x = PosX;
	mCoordinates.y = PosY;
}



void ColorSprite::setSize(int Width, int Height )
/******************************************************************************
 * sets a new Sprite-Size
 * Remembers the old position of the sprite by adding a new entry to
 * mRedrawPositions. These Positions will than be overdrawn with mRedrawColor
 * (meaning they'll be deleted from the Surface) on the next call of
 * ColorSprite.draw()
 *
 * 	PARAMETER
 * 		int PosX, PosY: new Coordinates of the sprite.
 *****************************************************************************/
{
	saveLastDrawnPosition();
	mCoordinates.w = Width;
	mCoordinates.h = Height;
}



// -------------------------
// private members
// -------------------------


void ColorSprite::redraw(SDL_Surface* Surface)
/*****************************************************************************
 * cancels the old images of the sprite from the surface, by redrawing the
 * last drawn sprite with the mRedrawColor
 * 	PARAMETER
 * 		SDL_Sprite* Surface: the Surface to draw into
 *****************************************************************************/
{
	if (mPositionChanged)
		SDL_FillRect(Surface, &mRedrawCoordinates, mRedrawColor);
}


 void ColorSprite::saveLastDrawnPosition()
/*****************************************************************************
 * saves the last position of the sprite that was drawn on the surface into
 * mRedrawCoordinates, the way it can later be overdrawn
 *****************************************************************************/
{
	if (!mPositionChanged)
	{
		mRedrawCoordinates.x = mCoordinates.x;
		mRedrawCoordinates.y = mCoordinates.y;
		mRedrawCoordinates.w = mCoordinates.w;
		mRedrawCoordinates.h = mCoordinates.h;
		mPositionChanged = true;
	}
}
